package Bencher::Scenarios::CloneModules;

our $DATE = '2021-07-31'; # DATE
our $VERSION = '0.050'; # VERSION

1;
# ABSTRACT: Scenarios related to data cloning modules

__END__

=pod

=encoding UTF-8

=head1 NAME

Bencher::Scenarios::CloneModules - Scenarios related to data cloning modules

=head1 VERSION

This document describes version 0.050 of Bencher::Scenarios::CloneModules (from Perl distribution Bencher-Scenarios-CloneModules), released on 2021-07-31.

=head1 DESCRIPTION

This distribution contains the following L<Bencher> scenario modules:

=over

=item * L<Bencher::Scenario::CloneModules>

=item * L<Bencher::Scenario::CloneModules::Startup>

=back

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Bencher-Scenarios-CloneModules>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Bencher-Scenarios-CloneModules>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Bencher-Scenarios-CloneModules>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 CONTRIBUTORS

=for stopwords perlancar (@pc-office) Steven Haryanto (on PC, Jakarta)

=over 4

=item *

perlancar (@pc-office) <perlancar@gmail.com>

=item *

Steven Haryanto (on PC, Jakarta) <stevenharyanto@gmail.com>

=back

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2021, 2017, 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
